/*
	File:		EI_GraphicsImport.r
	
	Description: Graphics importer component resources

	Author:		??

	Copyright: 	 Copyright 1999 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
				
	Change History (most recent first):

	   <1>	 	11/28/99	???		first file

*/

#define SystemSevenOrLater 1
#define UseExtendedThingResource	1

#include <ConditionalMacros.r>
#include <MacTypes.r>
#include <Components.r>
#include <QuickTimeComponents.r>
#include <ImageCompression.r>
#include "EI_GraphicsImportVersion.h"

#define kEI_GraphicsImportFlags	(graphicsImporterUsesImageDecompressor)

resource 'thng' (128) {
	'grip',
	'EIDI',
	'appl',
#if TARGET_REZ_MAC_68K
	kEI_GraphicsImportFlags,
	0,
	'grip', 
	128,
#else
	0,
	0,
	0,
	0,
#endif
	'STR ',
	128,
	0,
	0,
	0,
	0,
#ifdef TARGET_CPU_PPC
	kEI_GraphicsImportVersionPPC,
#else
	kEI_GraphicsImportVersion,
#endif
#ifdef	TARGET_CPU_PPC
	componentHasMultiplePlatforms + 
#endif
		componentDoAutoVersion,
	0,
	{
#if	TARGET_REZ_MAC_PPC
		kEI_GraphicsImportFlags, 
		'grpc',
		128,
		platformPowerPC,
#elif	!TARGET_OS_MAC
		kEI_GraphicsImportFlags, 
		'dlle',
		128,
		Target_PlatformType,
#endif
	},
};

resource 'thga' (129) {
	'grip', 
	'EIM ', // This must be in uppercase.  It will match an ".eim" suffix case-insensitively.
	'appl',
	kEI_GraphicsImportFlags | movieImportSubTypeIsFileExtension,
	0, 
	0, 
	0,
	'STR ',
	128,
	0,
	0, 
	0, 
	0,
	
	'grip',
	'EIDI',
	0, 0, 0
};

resource 'STR ' (128) {
	"Electric Image"
};

#if	!TARGET_OS_MAC
	resource 'dlle' (128) {
		"EI_GraphicsImportComponentDispatch"
	};
#endif

/* 
 * This is an example of how to build an atom container resource to hold mime types.
 * This component's GetMIMETypeList implementation simply loads this resource and returns it.
 * Please note that atoms of the same type MUST be grouped together within an atom container.
 * (Also note that "image/electric-image" may not have been registered with the IETF.)
 */
resource 'mime' (128) {
	{
	kMimeInfoMimeTypeTag,      1, "image/electric-image";
	kMimeInfoMimeTypeTag,      2, "image/x-electric-image";
	kMimeInfoFileExtensionTag, 1, "eim";
	kMimeInfoFileExtensionTag, 2, "eim";
	kMimeInfoDescriptionTag,   1, "Electric Image";
	kMimeInfoDescriptionTag,   2, "Electric Image";
	};
};
